@@ -13,49 +13,12 @@ using namespace nanotime;
 
 
 // for debug reasons...
-// the following code from: https://stackoverflow.com/a/16692519
+// cf https://stackoverflow.com/a/16692519
 template<typename Clock, typename Duration>
 std::ostream &operator<<(std::ostream &stream,
                          const std::chrono::time_point<Clock, Duration> &time_point) {
-  const time_t time = Clock::to_time_t(time_point);
-#if __GNUC__ > 4 || \
-   ((__GNUC__ == 4) && __GNUC_MINOR__ > 8 && __GNUC_REVISION__ > 1)
-    // Maybe the put_time will be implemented later?
-    struct tm tm;
-  localtime_r(&time, &tm);
-  return stream << std::put_time(&tm, "%c"); // Print standard date&time
-#else
-  char buffer[26];
-  ctime_r(&time, buffer);
-  buffer[24] = '\0';  // Removes the newline that is added
-  return stream << buffer;
-#endif
-}
-
-
-static inline duration getOffsetCnv(const dtime& dt, const std::string& z) {
-  int offset;
-  int res = RcppCCTZ::getOffset(std::chrono::duration_cast<std::chrono::seconds>(dt.time_since_epoch()).count(), z.c_str(), offset);
-  if (res < 0) {
-    Rcpp::stop("Cannot retrieve timezone '%s'.", z.c_str()); // ## nocov
-  }
-
-  return duration(offset).count() * std::chrono::seconds(1);
-}
-
-period::period() : months(0), days(0), dur(std::chrono::seconds(0)) { }
-
-period::period(int32_t months_p, int32_t days_p, duration dur_p) : months(months_p), days(days_p), dur(dur_p) {
-  // we have to ensure that NA is uniform across all construction
-  // possibilities so that functions that compare equality do so
-  // correctly:
-  if (months == std::numeric_limits<int32_t>::min() ||
-      days   == std::numeric_limits<int32_t>::min() ||
-      dur    == duration::min()) {
-    months = std::numeric_limits<int32_t>::min();
-    days = std::numeric_limits<int32_t>::min();
-    dur = duration::zero();
-  }
+  // updated with https://stackoverflow.com/a/71442312/23224962 and using C++17
+  return stream << std::chrono::duration_cast<std::chrono::nanoseconds>(time_point.time_since_epoch()).count();
 }
 
 
@@ -68,7 +31,13 @@ period::period(const std::string& str) {
   dur    = std::chrono::seconds(0);
 
   int n;
-  if (s < e && (*s == '/' || (s+2 < e && s[2] == ':'))) goto getduration;
+  if (s < e && (*s == '/' || (*s != '-' && s+2 < e && s[2] == ':'))) goto getduration;
+  // test the case where we have only a negative duration:
+  if (s < e && *s == '-' &&
+      ((s+3 < e && s[3] == ':') || (s+2 < e && s[2] == ':'))) {      
+    --s;  // because getduration will increment it
+    goto getduration;
+  }
   if (!readNumber(s, e, n, true) || s == e) throw std::range_error("cannot parse nanoperiod");
   if (*s == 'y') {
     months += 12*n;
@@ -135,72 +104,6 @@ std::string nanotime::to_string(const period& p) {
 }
 
 
-period nanotime::plus(const period&    p, duration d) {
-  return period(p.getMonths(), p.getDays(), p.getDuration() + d);
-}
-period nanotime::minus(const period&    p, duration d){
-  return period(p.getMonths(), p.getDays(), p.getDuration() - d);
-}
-period nanotime::minus(duration d, const period& p) {
-  return period(-p.getMonths(), -p.getDays(), -p.getDuration() + d);
-}
-
-dtime nanotime::plus(const dtime& dt, const period& p, const std::string& z) {
-  auto res = dt;
-  auto offset = getOffsetCnv(res, z);
-  if (p.getMonths()) {
-    auto dt_floor = date::floor<date::days>(dt + offset);
-    auto timeofday_offset = (dt + offset) - dt_floor;
-    auto dt_ymd = date::year_month_day{dt_floor};
-    dt_ymd += date::months(p.getMonths());
-    res = date::sys_days(dt_ymd) - offset + timeofday_offset;
-  }
-  offset = getOffsetCnv(dt, z);
-  res += p.getDays()*std::chrono::hours(24);
-  res += p.getDuration();
-  auto new_offset = getOffsetCnv(res, z);
-  // adjust for DST or any other event that changed the TZ, but only
-  // if the adjustment does not put us back in the old offset:
-  if (new_offset != offset) {
-    auto res_potential = res + offset - new_offset; // adjust
-    auto adjusted_offset = getOffsetCnv(res_potential, z);
-    if (adjusted_offset == new_offset) { // are we still in the new offset?
-      res = res_potential;               // if so, then keep the adjustment
-    }
-  }
-  return res;
-}
-
-dtime nanotime::minus(const dtime& dt, const period& p, const std::string& z) {
-  return plus(dt, -p, z);
-}
-
-interval nanotime::plus(const interval& i, const period& p, const std::string& z) {
-  return interval(plus(dtime{duration{i.s}}, p, z),
-                  plus(dtime{duration{i.e}}, p, z), i.sopen, i.eopen);
-}
-  
-interval nanotime::minus(const interval& i, const period& p, const std::string& z) {
-  return plus(i, -p, z);
-}
-
-
-period nanotime::operator+(const period& p1, const period& p2) {
-  return period(p1.getMonths()+p2.getMonths(), 
-                p1.getDays()+p2.getDays(), 
-                p1.getDuration()+p2.getDuration());
-}
-
-period nanotime::operator-(const period& p) {
-  return period(-p.getMonths(), -p.getDays(), -p.getDuration());
-}
-
-period nanotime::operator-(const period& p1, const period& p2) {
-  return period(p1.getMonths()-p2.getMonths(), 
-                p1.getDays()-p2.getDays(),
-                p1.getDuration()-p2.getDuration());
-}
-
 template <typename T>
 period nanotime::operator*(const period& p, T d) {
   return period(p.getMonths()*d, 
